@@ -14,10 +14,14 @@ import sys
 import struct
 import shutil
 import textwrap
-from bcolz.utils import to_ndarray
+import pickle
+import ctypes
+from distutils.version import LooseVersion
+import inspect
 
 import numpy as np
-from numpy.testing import assert_array_equal, assert_allclose
+from numpy.testing import (
+    assert_array_equal, assert_array_almost_equal, assert_allclose)
 from bcolz.tests import common
 from bcolz.tests.common import (
     MayBeDiskTest, TestCase, unittest, skipUnless, SkipTest)
@@ -25,9 +29,7 @@ import bcolz
 from bcolz.py2help import xrange, PY2, _inttypes
 from bcolz.carray_ext import chunk
 from bcolz import carray
-import pickle
-
-import ctypes
+from bcolz.utils import to_ndarray
 
 is_64bit = (struct.calcsize("P") == 8)
 
@@ -290,6 +292,17 @@ class getitemTest(MayBeDiskTest):
         # print "b[sl]->", `b[sl]`
         self.assertRaises(NotImplementedError, b.__getitem__, sl)
 
+    def test06(self):
+        """Testing `__getitem()__` with chunks not multiple of blocksize."""
+        a = np.arange(1e5)
+        b = bcolz.carray(a, chunklen=10000, rootdir=self.rootdir)
+        sl = -2   # second last element
+        # print "b[sl]->", `b[sl]`
+        assert_array_equal(a[sl], b[sl], "Arrays are not equal")
+        sl = -1   # last element
+        # print "b[sl]->", `b[sl]`
+        assert_array_equal(a[sl], b[sl], "Arrays are not equal")
+
 
 class getitemMemoryTest(getitemTest, TestCase):
     disk = False
@@ -337,7 +350,7 @@ class setitemTest(MayBeDiskTest):
         # print "b->", `b`
         assert_array_equal(a, b[:], "__setitem__ not working correctly")
 
-    def test03(self):
+    def test03a(self):
         """Testing `__setitem()__` method with the complete range"""
         a = np.arange(1e2)
         b = bcolz.carray(a, chunklen=10, rootdir=self.rootdir)
@@ -346,6 +359,16 @@ class setitemTest(MayBeDiskTest):
         # print "b->", `b`
         assert_array_equal(a, b[:], "__setitem__ not working correctly")
 
+    def test03b(self):
+        """Testing `__setitem()__` method with the complete range (large)"""
+        a = np.arange(1e6)
+        b = bcolz.carray(a, chunklen=1000-1)
+        c = bcolz.carray(a, chunklen=1000-2, rootdir=self.rootdir)
+        c[:] = b[:]
+        # print "b->", `b`
+        # print "c->", `c`
+        assert_array_equal(b[:], c[:], "__setitem__ not working correctly")
+
     def test04a(self):
         """Testing `__setitem()__` method with start:stop:step"""
         a = np.arange(1e2)
@@ -1446,15 +1469,15 @@ class evalTest(MayBeDiskTest):
     vm = "python"
 
     def setUp(self):
-        self.prev_vm = bcolz.defaults.eval_vm
+        self.prev_vm = bcolz.defaults.vm
         if bcolz.numexpr_here:
-            bcolz.defaults.eval_vm = self.vm
+            bcolz.defaults.vm = self.vm
         else:
-            bcolz.defaults.eval_vm = "python"
+            bcolz.defaults.vm = "python"
         MayBeDiskTest.setUp(self)
 
     def tearDown(self):
-        bcolz.defaults.eval_vm = self.prev_vm
+        bcolz.defaults.vm = self.prev_vm
         MayBeDiskTest.tearDown(self)
 
     def test00(self):
@@ -1551,12 +1574,9 @@ class evalTest(MayBeDiskTest):
     def test08(self):
         """Testing eval() via expression with lists (raise ValueError)"""
         a, b = range(int(self.N)), range(int(self.N))
-        depth = 3
-        if sys.version_info >= (3, 0):
-            depth += 1  # curiously enough, Python 3 needs one level more
-        self.assertRaises(ValueError, bcolz.eval, "a*3", depth=depth,
+        self.assertRaises(ValueError, bcolz.eval, "a*3", user_dict={'a':a,'b':b},
                           rootdir=self.rootdir)
-        self.assertRaises(ValueError, bcolz.eval, "b*3", depth=depth,
+        self.assertRaises(ValueError, bcolz.eval, "b*3", user_dict={'a':a,'b':b},
                           rootdir=self.rootdir)
 
     def test09(self):
@@ -1585,6 +1605,8 @@ class evalTest(MayBeDiskTest):
         c, d = bcolz.carray(a, rootdir=self.rootdir), bcolz.carray(b)
         if self.vm == "python":
             cr = bcolz.eval("np.sin(c) + 2 * np.log(d) - 3")
+        elif self.vm == "dask":
+            cr = bcolz.eval("da.sin(c) + 2 * da.log(d) - 3")
         else:
             cr = bcolz.eval("sin(c) + 2 * log(d) - 3")
         nr = np.sin(a) + 2 * np.log(b) - 3
@@ -1659,6 +1681,37 @@ class evalVeryBigNE(evalTest, TestCase):
     N = int(1e7)
     vm = "numexpr"
 
+@skipUnless(bcolz.dask_here, 'Needs dask')
+class evalSmallDSK(evalTest, TestCase):
+    N = 10
+    vm = "dask"
+
+
+@skipUnless(bcolz.dask_here, 'Needs dask')
+class evalDiskSmallDSK(evalTest, TestCase):
+    N = 10
+    vm = "dask"
+    disk = True
+
+
+@skipUnless(bcolz.dask_here, 'Needs dask')
+class evalBigDSK(evalTest, TestCase):
+    N = 1e4
+    vm = "dask"
+
+
+@skipUnless(bcolz.dask_here, 'Needs dask')
+class evalDiskBigDSK(evalTest, TestCase):
+    N = 1e4
+    vm = "dask"
+    disk = True
+
+
+@skipUnless(bcolz.dask_here and common.heavy, "Needs dask and --heavy")
+class evalVeryBigDSK(evalTest, TestCase):
+    N = int(1e7)
+    vm = "dask"
+
 
 class computeMethodsTest(TestCase):
 
@@ -2083,7 +2136,8 @@ class bloscCompressorsTest(MayBeDiskTest, TestCase):
 
     def tearDown(self):
         # Restore defaults
-        bcolz.cparams.setdefaults(clevel=5, shuffle=bcolz.SHUFFLE, cname='blosclz')
+        bcolz.cparams.setdefaults(clevel=5, shuffle=bcolz.SHUFFLE,
+                                  cname='blosclz', quantize=0)
         MayBeDiskTest.tearDown(self)
 
     def test00(self):
@@ -2131,7 +2185,8 @@ class bloscCompressorsTest(MayBeDiskTest, TestCase):
         # print "\nsize b uncompressed-->", a.size * a.dtype.itemsize
         for cname in cnames:
             bcolz.defaults.cparams = {
-                'clevel': 9, 'shuffle': bcolz.SHUFFLE, 'cname': cname}
+                'clevel': 9, 'shuffle': bcolz.SHUFFLE, 'cname': cname,
+                'quantize': 0}
             b = bcolz.carray(a, rootdir=self.rootdir)
             # print "size b compressed  -->", b.cbytes, "with '%s'"%cname
             self.assertTrue(sys.getsizeof(b) < b.nbytes,
@@ -2140,10 +2195,44 @@ class bloscCompressorsTest(MayBeDiskTest, TestCase):
             # Remove the array on disk before trying with the next one
             if self.disk:
                 common.remove_tree(self.rootdir)
-        # Restore defaults
-        bcolz.defaults.cparams = {
-            'clevel': 5, 'shuffle': bcolz.SHUFFLE, 'cname': 'blosclz'}
 
+    def test02a(self):
+        """Testing quantize filter on big arrays (float64)"""
+        np.random.seed(10)
+        a = np.cumsum(np.random.random_sample(100*1000)-0.5)    # random walk
+        if common.verbose:
+            print("Checking quantize filter")
+        # print "\nsize b uncompressed-->", a.size * a.dtype.itemsize
+        cparams = bcolz.cparams(quantize=0)
+        b = bcolz.carray(a, cparams=cparams, rootdir=self.rootdir)
+        b_cbytes = b.cbytes
+        assert_array_equal(a, b[:], "Arrays are not equal")
+        # print "size b compressed  -->", b_cbytes
+        # Remove the array on disk before trying with the next one
+        if self.disk:
+            common.remove_tree(self.rootdir)
+        cparams = bcolz.cparams(quantize=3)
+        c = bcolz.carray(a, cparams=cparams, rootdir=self.rootdir)
+        # print "size c compressed  -->", c.cbytes
+        self.assertTrue(c.cbytes < 0.7 * b_cbytes,
+                        "quantize does not seem to improve compression "
+                        "significantly")
+        assert_array_almost_equal(a, c[:], 3, "Arrays are not equal")
+        # Remove the array on disk before trying with the next one
+        if self.disk:
+            common.remove_tree(self.rootdir)
+
+    def test02b(self):
+        """Testing quantize filter on int arrays"""
+        a = np.arange(100*1000)
+        if common.verbose:
+            print("Checking quantize filter on ints")
+        cparams = bcolz.cparams(quantize=3)
+        self.assertRaises(TypeError, bcolz.carray, a, cparams=cparams,
+                          rootdir=self.rootdir)
+        # Remove the array on disk before trying with the next one
+        if self.disk:
+            common.remove_tree(self.rootdir)
 
 class compressorsMemoryTest(bloscCompressorsTest, TestCase):
     disk = False
@@ -2206,7 +2295,8 @@ class bloscFiltersTest(MayBeDiskTest, TestCase):
         # print "\nsize b uncompressed-->", a.size * a.dtype.itemsize
         for filter_ in filters:
             bcolz.defaults.cparams = {
-                'clevel': 9, 'shuffle': filter_, 'cname': "blosclz"}
+                'clevel': 9, 'shuffle': filter_, 'cname': "blosclz",
+                'quantize': 0}
             b = bcolz.carray(a, rootdir=self.rootdir)
             # print "size b compressed  -->", b.cbytes, "with '%s'"%cname
             self.assertTrue(sys.getsizeof(b) < b.nbytes,
@@ -2217,7 +2307,8 @@ class bloscFiltersTest(MayBeDiskTest, TestCase):
                 common.remove_tree(self.rootdir)
         # Restore defaults
         bcolz.defaults.cparams = {
-            'clevel': 5, 'shuffle': bcolz.SHUFFLE, 'cname': 'blosclz'}
+            'clevel': 5, 'shuffle': bcolz.SHUFFLE, 'cname': 'blosclz',
+            'quantize': 0}
 
 
 class filtersMemoryTest(bloscFiltersTest, TestCase):
@@ -2235,7 +2326,8 @@ class reprTest(TestCase):
         result = repr(ct)
         self.assertTrue("['2010-01-01' '2010-01-02']" in result)
 
-    @unittest.skipIf(os.name == "nt", "Windows problems with TZ here")
+    @unittest.skipIf(np.__version__ < LooseVersion("1.11"),
+                     "bcolz adapted to NumPy 1.11 (naive) TZ repr")
     def test_datetime_carray_nanos(self):
         x = ['2014-12-29T17:57:59.000000123',
              '2014-12-29T17:57:59.000000456']
@@ -2272,8 +2364,9 @@ class reprDiskTest(MayBeDiskTest,TestCase):
     def _create_expected(self, mode):
         expected = textwrap.dedent("""
                    carray((0,), float64)
-                     nbytes: 0; cbytes: 16.00 KB; ratio: 0.00
-                     cparams := cparams(clevel=5, shuffle=1, cname='blosclz')
+                     nbytes := 0; cbytes := 16.00 KB; ratio: 0.00
+                     cparams := cparams(clevel=5, shuffle=1, cname='blosclz', quantize=0)
+                     chunklen := 2048; chunksize: 16384; blocksize: 0
                      rootdir := '%s'
                      mode    := '%s'
                    []
@@ -2297,6 +2390,7 @@ class reprDiskTest(MayBeDiskTest,TestCase):
         expected = self._create_expected('a')
         self.assertEqual(expected, repr(y))
 
+
 class chunksIterTest(MayBeDiskTest):
     def test00(self):
         """Testing chunk iterator"""
